/**
	$Id: Main.java 651 2008-04-11 11:19:09Z nicklas $

	Copyright (C) Authors contributing to this file.

	This is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.
	
	The software is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.
	
	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.ws.example;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;

import org.apache.axis2.AxisFault;
import org.apache.commons.io.IOUtils;

import net.sf.basedb.info.DataFileTypeInfo;
import net.sf.basedb.info.ExperimentInfo;
import net.sf.basedb.info.FileInfo;
import net.sf.basedb.info.FileSetMemberInfo;
import net.sf.basedb.info.ProjectInfo;
import net.sf.basedb.info.QueryOptions;
import net.sf.basedb.info.RawBioAssayInfo;
import net.sf.basedb.info.VersionInfo;
import net.sf.basedb.ws.client.ExperimentClient;
import net.sf.basedb.ws.client.ProjectClient;
import net.sf.basedb.ws.client.RawBioAssayClient;
import net.sf.basedb.ws.client.SessionClient;

/**
	Example code for using BASE web services. This class need
	three arguments to it's main method:
	
	<pre>
	servicesUrl
	login
	password
	</pre>
	
	This class will connect to the BASE server as specified in the 
	<code>servicesUrl</code>, login, list all projects, experiments
	and raw bioassays the user has access to. It may optionally download
	data files associated with a raw bioassay. The final step is to logout
	again.

	@author nicklas
	@version 2.5
	@base.modified $Date: 2008-04-11 13:19:09 +0200 (Fri, 11 Apr 2008) $
*/
public class Main
{

	public static void main(String[] args)
	{
		if (args.length < 3 || args.length > 4) 
		{
			showUsage();
			System.exit(0);
		}
		String serviceUrl = args[0];
		String username = args[1];
		String password = args[2];
		String directory = args.length >= 4 ? args[3] : null;
		

		Main main = new Main();
		try
		{
			System.out.println("Connecting to: " + serviceUrl);
			main.connect(serviceUrl);
			System.out.println("------------");
			main.login(username, password);
			System.out.println("------------");
			main.listProjects();
			System.out.println("------------");
			main.listExperiments();
			System.out.println("------------");
			main.listRawBioassays(directory);
			System.out.println("------------");
			main.logout();
		}
		catch (Throwable t)
		{
			t.printStackTrace();
			System.exit(1);
		}
	}
	
	private static void showUsage()
	{
		System.out.println("Usage: Main <url> <login> <password> [directory]");
		System.out.println("       url       : The URL to the BASE webservices server");
		System.out.println("       login     : The login for an existing user account on the BASE server");
		System.out.println("       password  : The password for the account");
		System.out.println("       directory : Optional; if specified all raw bioassay data files will be downloaded to this directory");
	}
	
	private String serviceUrl;
	private SessionClient session;
	
	public Main()
	{}
	
	/**
		Connect to BASE with web services.
		@param serviceUrl The URL to the BASE webservices server
	*/
	public void connect(String serviceUrl)
		throws AxisFault
	{
		this.serviceUrl = serviceUrl;
		this.session = new SessionClient(serviceUrl, null, null);
		System.out.println("Connected to: " + serviceUrl);
		VersionInfo version = session.getVersion(); 
		System.out.println("BASE Version: " + version.toString());
	}
	
	/**
		Login to BASE.
		@param login The login for the account
		@param password The password
	*/
	public void login(String login, String password)
		throws AxisFault
	{
		session.login(login, password, null, false);
		int timeout = session.getSessionTimeOut();
		System.out.println("Logged in as: " + login);
		System.out.println("Timeout is: " + timeout + " minutes");
	}
	
	/**
		Logout from BASE.
	*/
	public void logout()
		throws AxisFault
	{
		session.logout();
		System.out.println("Logged out!");
	}

	/**
		List all projects that the logged in user is has access to.
		The first project found is made the active project.
	*/
	public void listProjects()
		throws AxisFault
	{
		ProjectClient pc = new ProjectClient(session);
		ProjectInfo[] projects = pc.getProjects(new QueryOptions());
		
		if (projects == null || projects.length == 0)
		{
			System.out.println("No projects found.");
		}
		else
		{
			System.out.println(projects.length + " projects found");
			int i = 0;
			while (i < projects.length)
			{
				ProjectInfo info = projects[i];
				++i;
				System.out.println( i + ". " + info.getName() + " [id=" + info.getId() + "]");
			}
			
			System.out.print("Activating project: " + projects[0].getName() + "...");
			pc.setActive(projects[0].getId());
			System.out.println("Done!");
		}
	}

	/**
		List all experiments that the logged in user is has access to.
	*/
	public void listExperiments()
		throws AxisFault
	{
		ExperimentClient ec = new ExperimentClient(session);
		ExperimentInfo[] experiments = ec.getExperiments(new QueryOptions());
		
		if (experiments == null || experiments.length == 0)
		{
			System.out.println("No experiments found.");
		}
		else
		{
			System.out.println(experiments.length + " experiments found");
			int i = 0;
			while (i < experiments.length)
			{
				ExperimentInfo info = experiments[i];
				++i;
				System.out.println( i + ". " + info.getName() + " [id=" + info.getId() + "]");
			}
		}
	}
	
	/**
		List all raw bioassays that the logged in user is has access to. For
		each raw bioassay, list it's datafiles and download those to the specified
		directory. If no directory is specified the download is disabled.
	*/
	public void listRawBioassays(String directory)
		throws AxisFault
	{
		RawBioAssayClient rc = new RawBioAssayClient(session);
		RawBioAssayInfo[] rawBioAssays = rc.getRawBioAssays(new QueryOptions());
		
		File downloadDir = directory != null ? new File(directory) : null;
		boolean doDownload = downloadDir != null && downloadDir.isDirectory();
		
		if (rawBioAssays == null || rawBioAssays.length == 0)
		{
			System.out.println("No raw bioassays found.");
		}
		else
		{
			System.out.println(rawBioAssays.length + " raw bioassays found; " + 
				(doDownload ? "Downloading to directory '" + directory + "'" : "File download is disabled"));
			int i = 0;
			while (i < rawBioAssays.length)
			{
				RawBioAssayInfo info = rawBioAssays[i];
				++i;
				System.out.println( i + ". " + info.getName() + " [id=" + info.getId() + "]");

				listDataFiles(rc, info, doDownload ? downloadDir : null);
			}
		}
	}

	/**
		List all datafiles for a given raw bioassay and download them if a 
		download directory is given.
	*/
	public void listDataFiles(RawBioAssayClient rc, RawBioAssayInfo info, File downloadDir)
		throws AxisFault
	{
		FileSetMemberInfo[] files = rc.getDataFiles(info.getId(), new QueryOptions());
		if (files == null || files.length == 0)
		{
			System.out.println("   No data files found.");
		}
		else
		{
			for (FileSetMemberInfo member : files)
			{
				DataFileTypeInfo fileType = member.getDataFileTypeInfo();
				FileInfo file = member.getFileInfo();
				
				System.out.println("   " + fileType.getName() + ": " + 
					file.getName() + " [id=" + file.getId() + "] " + file.getSize() + " bytes");
				if (downloadDir != null)
				{
					File downloadTo = new File(downloadDir, info.getName() + "." + file.getName());
					try
					{
						InputStream in = rc.downloadRawDataByType(info.getId(), fileType.getExternalId());
						OutputStream out = new FileOutputStream(downloadTo);
						IOUtils.copy(in, out);
						out.close();
						in.close();
						System.out.print("   Download --> " + downloadTo);
					}
					catch (IOException ex)
					{
						System.out.print("   Download failed: " + ex.getMessage());
					}
				}
				System.out.println("");
			}
		}
	}
	
}
